/* -------------------------------------------------------------------------*
 * Samples for the Attolic True Studio and the GCC                          *
 * Copyright (C) Lucjan Bryndza <lucjan.bryndza@ep.com.pl>  24.11.2010      *
 * Example No10 for the STM32 Butterfly2 and KAMODLCD1 and KAMODMEMS        *
 * Display temp on the N3310 disp                                           *
 *                                                                          *
 * http://www.boff.pl                                                       *
 * -------------------------------------------------------------------------*/

#include "stm32f10x_lib.h"
#include "system.h"
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include "time.h"
#include "nokia_lcd.h"
#include "i2c_master.h"
#include "tiny_printf.h"

/* ----------------------------------------------------------------------- */

//RGB I2c devaddr
#define I2C_SPEED 100000
#define LIS35_I2CADDR 58		/*I2c mems sensor addr */

#define LIS35_REG_OUTX 	0x29
#define LIS35_REG_OUTY	0x2B
#define LIS35_REG_OUTZ	0x2D

#define LIS35_REG_CR1 0x20
#define LIS35_REG_CR1_XEN 0x1
#define LIS35_REG_CR1_YEN 0x2
#define LIS35_REG_CR1_ZEN 0x4
#define LIS35_REG_CR1_DR_400HZ 0x80
#define LIS35_REG_CR1_ACTIVE 0x40
#define LIS35_REG_CR1_FULL_SCALE 0x20

#define LIS35_REG_CR2 0x21
#define LIS35_REG_CR2_BOOT 0x40

#define LIS35_CR3 0x22
#define LIS35_CR3_IHL 0x80
#define LIS35_CR3_CLICK_INT 0x7
#define LIS35_CR3_FF1_INT 0x1
#define LIS35_AUTOINC 0x80

#define LIS35_FF_WU_CFG_1 0x30
#define LIS35_FF_WU_SRC_1 0x31
#define LIS35_FF_WU_THS_1 0x32
#define LIS35_FF_WU_DURATION_1 0x33


#define LIS35_CLICK_CFG 0x38
#define LIS35_CLICK_THSY_X 0x3b
#define LIS35_CLICK_THSZ 0x3c
#define LIS35_CLICK_TIME_LIMIT 0x3D


#define LIS35_STATUS_REG 0x27

/* ------------------------------------------------------------------ */
//Configure temperature sensor to 12 bit resolution
static int lis35_init(void)
{
    
    static const uint8_t init_val[] =
    {
    	LIS35_REG_CR1, //Reg addr
        LIS35_REG_CR1_XEN | LIS35_REG_CR1_YEN | LIS35_REG_CR1_ZEN | LIS35_REG_CR1_ACTIVE //Reg Val
    };
    return i2cm_transfer_7bit( LIS35_I2CADDR, init_val, sizeof(init_val), NULL, 0 );
}

/* ------------------------------------------------------------------ */
//Pos def
typedef int pos_t; 

/* ------------------------------------------------------------------ */

//Get current temperature and fill the structure
static int lis35_get(pos_t *x, pos_t *y, pos_t *z)
{
    static const uint8_t outx_reg = LIS35_REG_OUTX | LIS35_AUTOINC;
    signed char r_regs[5];
    int res = i2cm_transfer_7bit( LIS35_I2CADDR, &outx_reg , sizeof(outx_reg), r_regs, sizeof(r_regs) );
    if(!res)
    {
    	*x = r_regs[0];
        *y = r_regs[2];
        *z = r_regs[4];
    }
    return res;
} 

/* ----------------------------------------------------------------------- */
//Display temperature
static void print_value(pos_t x,pos_t y, pos_t z)
{
   char buf[24]; 
   tiny_snprintf( buf,sizeof(buf),"X=%d   ",x);
   nlcd_put_string(buf,0,1);
   tiny_snprintf( buf,sizeof(buf),"Y=%d   ",y);
   nlcd_put_string(buf,0,2);
   tiny_snprintf( buf,sizeof(buf),"Z=%d   ",z);
   nlcd_put_string(buf,0,3);
}

/* ----------------------------------------------------------------------- */
//Print error code
static void print_error(int err)
{
    char buf[24];
    tiny_snprintf( buf, sizeof(buf), "I2cErr %d",err);
    nlcd_put_string(buf,0,1);
}

/* ----------------------------------------------------------------------- */
#define MEASURE_DELAY_TIME 200

int main(void)
{

    //Initialize system timer
    time_init();
    //Initialize nokia display
    nlcd_init();
    //Initialize the i2c module
    i2cm_init(I2C_SPEED);
    //Initialize temp controller
    int res = lis35_init();
    if(res<0)
    {
        print_error(res);
        for(;;);    //Fail never end loop
    }
    else
    {
        nlcd_put_string( "www.boff.pl", 0, 0 );
    }
    for(pos_t x=0,y=0,z=0;;)
    {        
	res = lis35_get(&x,&y,&z);
        if(res>=0)
        {
           print_value(x,y,z);
        }
        else
        {
          print_error(res);
          for(;;);      //Failed infinite loop
        }
        wait_ms(MEASURE_DELAY_TIME);
    }
    return 0;
}

/* ----------------------------------------------------------------------- */

